package com.webproglabs.lab1.lab34.services;

import com.webproglabs.lab1.lab34.controller.ProfileDto;
import com.webproglabs.lab1.lab34.jwt.JwtException;
import com.webproglabs.lab1.lab34.jwt.JwtProvider;
import com.webproglabs.lab1.lab34.model.Comment;
import com.webproglabs.lab1.lab34.model.Post;
import com.webproglabs.lab1.lab34.model.Profile;
import com.webproglabs.lab1.lab34.model.UserRole;
import com.webproglabs.lab1.lab34.repository.ProfileRepository;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityNotFoundException;
import javax.transaction.Transactional;
import java.util.*;

@Service
public class ProfileService implements UserDetailsService {

    private final ProfileRepository profileRepository;
    private final PasswordEncoder passwordEncoder;

    private final JwtProvider jwtProvider;

    public ProfileService(ProfileRepository profileRepository, PasswordEncoder passwordEncoder, JwtProvider jwtProvider) {
        this.profileRepository = profileRepository;
        this.passwordEncoder = passwordEncoder;
        this.jwtProvider = jwtProvider;
    }

    @Transactional
    public Profile findUser(Long id) {
        final Optional<Profile> profile = profileRepository.findById(id);
        return profile.orElseThrow(EntityNotFoundException::new);
    }

    @Transactional
    public Profile findUserByLogin(String login) {
        final Optional<Profile> profile = profileRepository.findByLoginLike(login).stream().findFirst();
        return profile.orElseThrow(EntityNotFoundException::new);
    }

    @Transactional
    public List<Profile> findAllUsers() {
        return profileRepository.findAll();
    }

    @Transactional
    public Profile addUser(String login, String password, List<Comment> comments, List<Post> posts) {
        if (!StringUtils.hasText(login) || !StringUtils.hasText(password)) {
            throw new IllegalArgumentException("User data is null or empty");
        }
        final Profile user = new Profile(login, password, comments, posts, UserRole.USER);
        return profileRepository.save(user);
    }

    @Transactional
    public Profile addUser(String login, String password, List<Comment> comments, List<Post> posts, UserRole role) {
        if (!StringUtils.hasText(login) || !StringUtils.hasText(password)) {
            throw new IllegalArgumentException("User data is null or empty");
        }
        final Profile user = new Profile(login, password, comments, posts, role);
        return profileRepository.save(user);
    }

    @Transactional
    public Profile createUser(String login, String password, String passwordConfirm) throws Exception {
        if (findByLogin(login) != null) {
            throw new Exception("User " + login + " already exists");
        }
        final Profile user = new Profile(login, passwordEncoder.encode(password), new ArrayList<>(), new ArrayList<>(), UserRole.USER);
        if (!Objects.equals(password, passwordConfirm)) {
            throw new Exception("Passwords not equals");
        }
        return profileRepository.save(user);
    }

    @Transactional
    public Profile createUser(String login, String password, String passwordConfirm, UserRole role) throws Exception {
        if (findByLogin(login) != null) {
            throw new Exception("User " + login + " already exists");
        }
        final Profile user = new Profile(login, passwordEncoder.encode(password), new ArrayList<>(), new ArrayList<>(), role);
        if (!Objects.equals(password, passwordConfirm)) {
            throw new Exception("Passwords not equals");
        }
        return profileRepository.save(user);
    }

    @Transactional
    public Profile updateUser(Long id, String login, String password) {
        if (!StringUtils.hasText(login) || !StringUtils.hasText(password)) {
            throw new IllegalArgumentException("User data is null or empty");
        }
        final Profile currentUser = findUser(id);
        currentUser.setLogin(login);
        currentUser.setPassword(password);
        return profileRepository.save(currentUser);
    }

    @Transactional
    public Profile deleteUser(Long id) {
        final Profile currentUser = findUser(id);
        profileRepository.delete(currentUser);
        return currentUser;
    }

    @Transactional
    public void deleteAllUsers() {
        profileRepository.deleteAll();
    }

    public Profile findByLogin(String login) {
        return profileRepository.findOneByLoginIgnoreCase(login);
    }
    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        final Profile userEntity = findByLogin(username);
        if (userEntity == null) {
            throw new UsernameNotFoundException(username);
        }
        return new org.springframework.security.core.userdetails.User(
                userEntity.getLogin(), userEntity.getPassword(), Collections.singleton(userEntity.getRole()));
    }

    public String loginAndGetToken(ProfileDto userDto) {

        try {
            final Profile user = findByLogin(userDto.getLogin());
            if (user == null) {
                throw new Exception("Login not found" + userDto.getLogin());
            }
            if (!passwordEncoder.matches(userDto.getPassword(), user.getPassword())) {
                throw new Exception("User not found" + user.getLogin());
            }
            return jwtProvider.generateToken(user.getLogin());
        }
        catch (Exception e) {
            var ex = e;
            return null;
        }

    }

    public UserDetails loadUserByToken(String token) throws UsernameNotFoundException {
        if (!jwtProvider.isTokenValid(token)) {
            throw new JwtException("Bad token");
        }
        final String userLogin = jwtProvider.getLoginFromToken(token)
                .orElseThrow(() -> new JwtException("Token is not contain Login"));
        return loadUserByUsername(userLogin);
    }
}
